package org.rzo.yacron4j.example;

import java.text.ParseException;
import java.util.Date;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import org.rzo.yacron4j.Scheduler;
import org.rzo.yacron4j.SchedulerOptions;
import org.rzo.yacron4j.TaskOptions;

public class SingleThreadExecution
{

	/**
	 * @param args
	 * @throws ParseException
	 */
	public static void main(String[] args) throws Exception
	{
		SchedulerOptions schedulerOptions = new SchedulerOptions();
		schedulerOptions.setMaxThreads(1);
		schedulerOptions.setMaxQueueSize(2); // should not be lower than the
												// number of parallel different
												// scheduled tasks. here 1 ->
												// will cause a
												// RejectedExecutionException.
		Scheduler scheduler = new Scheduler(schedulerOptions);
		String cronExpression = "0/1 * * * * ?";
		TaskOptions options = new TaskOptions();
		options.setAllowOverlap(true); // allow execution of same task to
										// overlap
		options.setMaxConcurrent(20); // we will still have no overlap because
										// we have only one thread
		ScheduledFuture task = scheduler.schedule(new Runnable()
		{

			public void run()
			{
				System.out.println("1 " + new Date() + " running "
						+ Thread.currentThread().getName());
				try
				{
					Thread.sleep(5000);
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}, cronExpression, options);
		ScheduledFuture task2 = scheduler.schedule(new Runnable()
		{

			public void run()
			{
				System.out.println("2 " + new Date() + " running "
						+ Thread.currentThread().getName());
				try
				{
					Thread.sleep(5000);
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}, cronExpression, options);

		System.out.println(new Date(System.currentTimeMillis()
				+ task.getDelay(TimeUnit.MILLISECONDS)));
		Thread.sleep(15000);
		scheduler.shutdown();
	}

}
