package org.rzo.yacron4j.persist.prevayler;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.Serializable;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.util.Date;

import org.prevayler.Prevayler;
import org.prevayler.PrevaylerFactory;
import org.prevayler.Transaction;
import org.rzo.yacron4j.persist.PersistManager;


public class PrevaylerPersistentManager implements PersistManager
{
	final Prevayler<Root> _prevayler;
	long _counter = 0;
	final String _folder;
	FileLock _lck;
	
	public PrevaylerPersistentManager(String folder) throws Exception
	{
	    // Create or load existing prevalence layer from journal and/or snapshot.
		_folder = folder;
		lockFolder();
		_prevayler = PrevaylerFactory.createPrevayler(new Root(), folder);
		takeSnapshot();

	}

	private void lockFolder() throws IOException
	{
		File f = new File(new File(_folder), "prevayler.lck");
		if (!f.getParentFile().exists())
			f.getParentFile().mkdirs();
		if (!f.exists())
			f.createNewFile();
		f.deleteOnExit();
		RandomAccessFile raf = new RandomAccessFile(f, "rw");
		FileChannel fc = raf.getChannel();
		_lck = fc.tryLock();
		if (_lck == null)
			throw new IOException("prevayler folder '"+_folder+ "' already locked");
	}

	public void scheduled(String id, long schedule)
	{
		_prevayler.execute(new ScheduledTransction(id, schedule));
		if (_counter++ > 5000)
			takeSnapshot();
	}

	private void takeSnapshot()
	{
		try
		{
			File f = _prevayler.takeSnapshot();
			cleanup(f);
			_counter = 0;
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void cleanup(File f)
	{
		long currentVersion = getVersion(f);
		File[] files = new File(_folder).listFiles();
		for (File file : files)
		{
			if (file.getName().endsWith(".journal") || file.getName().endsWith(".snapshot")
			     && (getVersion(file) < currentVersion))
				file.delete();
		}
	}
	
	private long getVersion(File f)
	{
		String name = f.getName();
		String version = name.substring(0, name.indexOf('.'));
		long result = Long.MAX_VALUE;
		try
		{
				result = Long.parseLong(version);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		return result;
	}

	public void terminated(String id)
	{
		_prevayler.execute(new TerminatedTransaction(id));
		if (_counter++ > 5000)
			takeSnapshot();
	}

	public Long getSchedule(String id)
	{
		return _prevayler.prevalentSystem().getSchedules().get(id);
	}

}
